/*
 * Tencent is pleased to support the open source community by making BK-JOB蓝鲸智云作业平台 available.
 *
 * Copyright (C) 2021 THL A29 Limited, a Tencent company.  All rights reserved.
 *
 * BK-JOB蓝鲸智云作业平台 is licensed under the MIT License.
 *
 * License for BK-JOB蓝鲸智云作业平台:
 * --------------------------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of
 * the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
 * THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package com.tencent.bk.job.execute.engine.result;

import java.util.concurrent.atomic.AtomicInteger;

/**
 * 文件任务结果处理调度策略
 */
public class FileTaskResultHandleScheduleStrategy implements ScheduleStrategy {
    /**
     * 任务累计执行次数
     */
    private volatile AtomicInteger times = new AtomicInteger(0);

    @Override
    public long getDelay() {
        int handleCount = times.addAndGet(1);
        if (handleCount <= 2) {
            // 2s以内，周期为1s
            return 1000;
        } else if (handleCount <= 11) {
            // 2s-20s,周期为2s
            return 2000;
        } else if (handleCount <= 67) {
            // 20s-5min,周期为5s
            return 5000;
        } else {
            // 超过5min,周期为10s
            return 10000;
        }
    }
}
